# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import logging
import jenkins
import os
import os.path

log = logging.getLogger(__name__)


def get_config():
    '''Load the jenkins configuration data created by the restish charm.'''
    config = None
    try:
        config_file_path = os.path.join(os.path.dirname(__file__),
                                        '../../lander_config.json')
        if os.path.exists(config_file_path):
            with open(config_file_path) as config_file:
                config = json.loads(config_file.read())
        else:
            log.error('No lander_config.json found at: '
                      '{}'.format(os.path.abspath(config_file_path)))
    except:
        log.exception('ERROR detecting lander-jenkins config')
    return config


def trigger_build(jenkins_config, job_parameters):
    '''Triggers a build on the target jenkins instance.'''
    jenkins_url = "http://{}:{}/".format(jenkins_config['lander_hostname'],
                                         jenkins_config['lander_port'])
    jenkins_handle = jenkins.Jenkins(jenkins_url,
                                     jenkins_config['lander_username'],
                                     jenkins_config['lander_password'])
    job_name = jenkins_config['lander_master_job']
    job_token = jenkins_config['lander_job_token']
    jenkins_handle.build_job(job_name, job_parameters, job_token)
    # The python jenkins api does not provide a status return, so return
    # None indicating that nothing bad happened so far.
    return None
