# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

import mock

from ci_utils import jenkins_utils


class TestJenkins(unittest.TestCase):
    @mock.patch('os.path.exists')
    def test_get_config_no_file(self, os_path_exists_mock):
        '''Assert that a missing config file returns None.'''
        os_path_exists_mock.return_value = False
        result = jenkins_utils.get_config()
        self.assertIsNone(result)

    @mock.patch('os.path.exists')
    @mock.patch('__builtin__.open')
    def test_get_config_file_error(self, open_mock, os_path_exists_mock):
        '''Assert that failure to open the config file returns None.'''
        file_mock = mock.Mock()
        file_mock.read.return_value = False
        context_manager_mock = mock.Mock()
        context_manager_mock.__enter__ = mock.Mock()
        context_manager_mock.__enter__.return_value = file_mock
        context_manager_mock.__exit__ = mock.Mock()
        open_mock.return_value = context_manager_mock
        os_path_exists_mock.return_value = True
        result = jenkins_utils.get_config()
        self.assertIsNone(result)

    @mock.patch('jenkins.Jenkins')
    def test_trigger_build(self, jenkins_mock):
        '''Test that a valid jenkins config can trigger a build.'''
        job_name = 'master_job'
        job_params = {'foo': 'bar'}
        job_token = 'token'
        jenkins_config = {'lander_hostname': 'http://localhost/',
                          'lander_port': 8080,
                          'lander_username': 'lander-bot',
                          'lander_password': 'lander-bot',
                          'lander_master_job': job_name,
                          'lander_job_token': job_token}
        build_job_mock = mock.Mock()
        build_job_mock.return_value = None
        jenkins_obj_mock = mock.Mock()
        jenkins_obj_mock.build_job = build_job_mock
        jenkins_mock.return_value = jenkins_obj_mock
        result = jenkins_utils.trigger_build(jenkins_config, job_params)
        build_job_mock.assert_called_once_with(job_name, job_params, job_token)
        self.assertIsNone(result)
