# Ubuntu Continuous Integration Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

from dput.changes import Changes, parse_changes_file
from os import path


class ChangesProcessor:
    """Class to parse, validate and upload source files to the datastore."""

    def __init__(self, changes_filepath, files_to_upload_dir=None):
        # Needs to be provided with the full path to the changes file.
        if not path.exists(changes_filepath):
            raise IOError("%s not found" % changes_filepath)
        self.changes_filepath = changes_filepath

        if files_to_upload_dir is not None:
            if not path.exists(files_to_upload_dir):
                raise IOError("%s not found" % files_to_upload_dir)
            self.files_to_upload_dir = files_to_upload_dir
        else:
            self.files_to_upload_dir = path.dirname(changes_filepath)
        self.files_to_upload = []
        self.changes = None

    def validate(self, check_signature=False):
        changes = self.changes
        try:
            # We don't care if signatures fail for now
            changes.validate(check_signature=check_signature)
            self.files_to_upload = [path.basename(file_) for file_ in
                                    changes.get_files()]
            self.files_to_upload.append(changes.get_filename())
        except IOError, exc:
            # File not found. Aborting upload
            raise exc

    def parse(self):
        try:
            self.changes = parse_changes_file(
                filename=self.changes_filepath,
                directory=self.files_to_upload_dir)
        except Exception, exc:
            raise exc

    def upload_files(self):
        for file_ in self.files_to_upload:
            # Do something
            pass

    def process(self, check_signature=False):
        self.parse()
        self.validate(check_signature=check_signature)
        self.upload_files()
