# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import logging

from restish import http, resource

from ci_utils import restish_utils

log = logging.getLogger(__name__)


def _build_image(base_image, ppa_list, package_list):
    return http.ok([('Content-Type', 'application/json')],
                   json.dumps(base_image))


def _status():
    return http.ok([('Content-Type', 'application/json')], "{}")


class API(resource.Resource):

    @resource.child('status')
    def status(self, request, segments):
        log.debug('[status]: %s %s', request.url, str(segments))
        return restish_utils.http_get_resource(_status)

    @resource.child()
    def build_image(self, request, segments):
        log.debug('[build_image]: %s %s', request.url,
                  json.dumps(request.body))
        return restish_utils.http_post_resource(_build_image)

    def _return_json(self, request):
        try:
            params = json.loads(request.body)
        except ValueError:
            return http.bad_request()
        return http.ok([('Content-Type', 'application/json')],
                       json.dumps(params))
