#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import json
import urllib2


def parse_arguments():
    parser = argparse.ArgumentParser(
        description='Archives the provided job artifacts to the data store.')
    parser.add_argument('--build-url',
                        required=True,
                        help='The build to archive.')
    return parser.parse_args()


def _get_build_api_json_url(url):
    return '/'.join([url, 'api', 'json'])


def _get_artifact_list(url):
    req = urllib2.Request(_get_build_api_json_url(url))
    f = urllib2.urlopen(req)
    response = f.read()
    f.close()
    data = json.loads(response)
    artifact_list = []
    for artifact in data['artifacts']:
        artifact_list.append(artifact['relativePath'])

    return artifact_list


def _get_console_log_url(url):
    return '/'.join([url, 'consoleText'])


def _get_artifact_url(url, artifact_name):
    return '/'.join([url, 'artifact', artifact_name])


def _dump_artifact(url):
    req = urllib2.Request(url)
    f = urllib2.urlopen(req)
    response = f.read()
    f.close()
    print(response)


def main():
    args = parse_arguments()

    #TODO: Do something useful with the artifacts.
    #TODO: For now, just dump them to the console.
    print('Console Log:')
    print('************')
    _dump_artifact(_get_console_log_url(args.build_url))
    artifact_list = _get_artifact_list(args.build_url)
    for artifact in artifact_list:
        print('Artifact: {}'.format(artifact))
        print('**********' + ('*' * len(artifact)))
        _dump_artifact(_get_artifact_url(args.build_url, artifact))


if __name__ == '__main__':
    main()
