# Ubuntu CI Engine
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import logging
import urllib
import urllib2

from django.conf import settings
from oauth import oauth


def lp_collection(url):
    '''iterate through each item returned in an lp_collection'''
    resp = urllib2.urlopen(url)
    resp = resp.read()
    data = json.loads(resp)

    for d in data['entries']:
        yield d

    if 'next_collection_link' in data:
        for d in lp_collection(data['next_collection_link']):
            yield d


def _oauth_headers():
    '''oauth header required for a launchpad api post

    https://help.launchpad.net/API/SigningRequests
    '''
    values = [
        'OAuth realm="%s"' % settings.OAUTH_REALM,
        'oauth_version="1.0"',
        'oauth_consumer_key="%s"' % settings.OAUTH_CONSUMER_KEY,
        'oauth_token="%s"' % settings.OAUTH_TOKEN,
        'oauth_signature_method="PLAINTEXT"',
        'oauth_signature="%%26%s"' % settings.OAUTH_TOKEN_SECRET,
        'oauth_timestamp="%d"' % int(oauth.time.time()),
        'oauth_nonce="%s"' % oauth.generate_nonce(),
    ]
    return {
        'Authorization': ', '.join(values),
        'Accept': 'application/json',
        'Content-Type': 'application/x-www-form-urlencoded'
    }


def _lp_post(url, data):
    req = urllib2.Request(url, urllib.urlencode(data), _oauth_headers())
    return urllib2.urlopen(req)


def get_publishing_history(ppa):
    '''A generator for the source and binary publish history of a ppa.'''
    url = settings.LAUNCHPAD_API_BASE + '/~%s/+archive/%s/?ws.op='
    url = url % (settings.LAUNCHPAD_PPA_USER, ppa.split('/')[-1])

    for x in lp_collection(url + 'getPublishedSources'):
        yield x
    for x in lp_collection(url + 'getPublishedBinaries'):
        yield x


def request_ppa_clean(ppa):
    '''call requestDeletion operation on ppa's source and binary pub history'''
    for x in get_publishing_history(ppa):
        if x['date_removed']:
            continue
        logging.debug('requesting deletion of: %s', x['self_link'])
        resp = _lp_post(x['self_link'], {'ws.op': 'requestDeletion'})
        code = resp.getcode()
        if code != 200:
            raise urllib2.HTTPError(
                x['self_link'], code, resp.read(), resp.headers.headers, None)


def _sync_source(ppa, source_name, version, pocket, archive):
    '''Used for populating a PPA so we can run integration tests.'''
    url = 'https://api.launchpad.net/1.0/~%s/+archive/%s'
    url = url % (settings.LAUNCHPAD_PPA_USER, ppa.split('/')[-1])
    data = {
        'ws.op': 'syncSource',
        'from_archive': archive,
        'source_name': source_name,
        'version': version,
        'to_pocket': pocket,
    }
    resp = _lp_post(url, data)
    c = resp.getcode()
    if c != 200:
        raise urllib2.HTTPError(url, c, resp.read(), resp.header.headers, None)
