#!/usr/bin/env python

import json
import os
import unittest

import urllib2
import yaml

import amulet

opd = os.path.dirname

root_dir = opd(opd(opd((os.path.abspath(__file__)))))


class AmuletDeployment(object):
    """Fixture for a deployed set of juju services from a deployer config"""

    def __init__(self, deployer_cfg, juju_env=None, sentries=False):
        self.deployment = amulet.Deployment(juju_env, sentries=sentries)
        with open(deployer_cfg) as f:
            jd_script = yaml.safe_load(f.read())
        # Use juju_env given or defaulted above rather than taking the name
        # from the deployer config and assuming an env of that name exists
        script = {self.deployment.juju_env: jd_script.values()[0]}
        self.deployment.load(script)

    def setUp(self):
        self.deployment.setup()


class JujuGuiTest(unittest.TestCase):
    """Integration tests for juju-gui service run on a juju deployment"""

    def setUp(self):
        super(JujuGuiTest, self).setUp()
        self.deployer = AmuletDeployment(os.path.join(root_dir, 'juju-deployer',
                                                      'juju-gui.yaml'))
        self.deployer.setUp()

    def get_unit(self):
        status = amulet.waiter.status(self.deployer.deployment.juju_env)
        unit = status['services']['ci-juju-gui']['units'].values()[0]
        return unit

    def test_juju_gui(self):
        """Ensure that the juju-gui is on the bootstrap node (0), and that
        there's an HTTP server listening on its address"""

        unit = self.get_unit()
        self.assertEqual('0', unit['machine'])

        url = 'http://%s' % unit['public-address']
        urllib2.urlopen(url).read()


if __name__ == "__main__":
    unittest.main()
